/*
Copyright 2017 Detelina Stoyanova

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
 */

package forAge;
import java.io.*;
import java.awt.Color;
import java.awt.Font;
import java.awt.event.*;
import java.nio.charset.*;
import java.awt.*;

import javax.imageio.ImageIO;
import javax.swing.*;
import javax.swing.filechooser.*;
import java.text.DecimalFormat;

/**
 * This function creates the menu and allows the user to interact with the program
 * 
 * @author Detelina Stoyanova
 * @version 20180217
 * 
 */
public class Menu implements ActionListener, ItemListener 
{
	
	private ForAge forAge;
	
	private JFrame frame = null;
	private JEditorPane pane = null;
	private JFrame aboutFrame = null;
	private JEditorPane aboutPane = null;
	
	private JButton okButton = null;
	private JButton flipButton = null;
	
	private JFrame isFlippedFrame = null;
	private JEditorPane isFlippedPane = null;
	
	private JMenuItem fileOpen = null;
	private JMenuItem fileSave = null;
	private JMenuItem exitJF = null;

	private JMenuItem savePCA = null;
	private JMenuItem saveCurvatureMesh = null;
	private JMenuItem saveTPSMesh = null;
	
	private JMenuItem about = null;
	
	private JCheckBoxMenuItem viewPCAMesh = null;
	private JCheckBoxMenuItem viewInitMesh = null;
	private JCheckBoxMenuItem viewTPSMesh = null;
	private JCheckBoxMenuItem viewCurvatureMesh = null;
	
	private String currentDirectory = "..";


	private String filePath;
	
	private forAgeVis vis;

	public void createMenuWindow() throws IOException 
	{	
		frame = new JFrame("forAge 20180217");
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		frame.setSize(500,500);
		pane = new JEditorPane();
		
		Font font = new Font("TimesRoman", Font.BOLD, 12);
		pane.setFont(font);
		
		pane.setEditable(false);
		
		pane.setText("\n Welcome to forAge. \n\n Please use the menu to select a PLY file.");
		
		frame.setContentPane(pane);
		
		try {
			frame.setIconImage(ImageIO.read(new File("icon/icon.png")));
		} catch (IOException e) {
		    // TODO Auto-generated catch block
		    e.printStackTrace();
		 }
		
		JMenuBar options = new JMenuBar();
		JMenu menu1 = getFileMenu();
		JMenu menu2 = getViewMenu();
		JMenu menu3 = getHelpMenu();
		options.add(menu1); 
		options.add(menu2); 
		options.add(menu3); 
		frame.setJMenuBar(options);
		
		frame.setVisible(true);
		
		vis = new forAgeVis();
        vis.run();
	}
   
	private JMenu getFileMenu() 
	{
		JMenu file = new JMenu("File");
		
		fileOpen = new JMenuItem("Open");
		fileOpen.addActionListener(this);
		file.add(fileOpen);
		
		fileSave = new JMenu("Save");
		savePCA = new JMenuItem("Positioned Scan");
		savePCA.addActionListener(this);
		fileSave.add(savePCA);
		saveCurvatureMesh = new JMenuItem("Ventral Outline Mesh");
		fileSave.add(saveCurvatureMesh);
		saveCurvatureMesh.addActionListener(this);
		saveTPSMesh = new JMenuItem("TPS Mesh");
		fileSave.add(saveTPSMesh);
		saveTPSMesh.addActionListener(this);
		fileSave.setEnabled(false);
		file.add(fileSave);
		
		exitJF = new JMenuItem("Exit");
		exitJF.addActionListener(new ActionListener() {
	        public void actionPerformed(ActionEvent event) {
	            System.exit(0);
	        }
	    });
		file.add(exitJF);
		
		return file;
	}
	
	private JMenu getViewMenu() 
	{
		JMenu view = new JMenu("View");
		
		viewPCAMesh = new JCheckBoxMenuItem("Positioned Scan");
		viewPCAMesh.addActionListener(this);
		viewPCAMesh.setEnabled(false);
		view.add(viewPCAMesh);
		
		view.addSeparator();
		
		viewInitMesh = new JCheckBoxMenuItem("Rectangular Mesh");
		viewInitMesh.addActionListener(this);
		viewInitMesh.setEnabled(false);
		view.add(viewInitMesh);
		
		view.addSeparator();
		
		viewTPSMesh = new JCheckBoxMenuItem("TPS Mesh");
		viewTPSMesh.addActionListener(this);
		viewTPSMesh.setEnabled(false);
		view.add(viewTPSMesh);
		
		view.addSeparator();
		
		viewCurvatureMesh = new JCheckBoxMenuItem("Ventral Outline Mesh");
		viewCurvatureMesh.addActionListener(this);
		viewCurvatureMesh.setEnabled(false);
		view.add(viewCurvatureMesh);
		
		return view;
	}
	
	private JMenu getHelpMenu() 
	{
		JMenu help = new JMenu("Help");
		
		about = new JMenuItem("About");
		about.addActionListener(this);
		help.add(about);
		
		return help;
	}
   
	public void actionPerformed(ActionEvent e) 
	{		
		JFileChooser jfc = new JFileChooser();
		jfc.setCurrentDirectory(new File(currentDirectory));
		jfc.setFileSelectionMode(JFileChooser.FILES_ONLY);
		
		FileNameExtensionFilter fileTypePLY = new FileNameExtensionFilter(".ply files", "ply");
		FileNameExtensionFilter fileTypeOBJ = new FileNameExtensionFilter(".obj files", "obj");
		jfc.setFileFilter(fileTypePLY);
		
		final DecimalFormat fmt = new DecimalFormat("##.00");
		final DecimalFormat fmt1 = new DecimalFormat("##.0000");
		
		Object option = e.getSource();
		try 
		{
			if (option == fileOpen) 
			{
				int file = jfc.showOpenDialog(pane);
				if (file == JFileChooser.APPROVE_OPTION) 
				{
					File selectedFile = jfc.getSelectedFile();
					filePath = selectedFile.getPath();
					
					/*create instance of forAge for the selected ply file*/
					forAge = new ForAge(filePath);
					pane.setText("You are using file: \n" + filePath + "\n\n" + "Follow the instructions in the pop-up window to position the scan." );
					
					currentDirectory = filePath;
					
					vis.setPCAVertices(forAge.getPCAVerticesArray());
					vis.setPCAFaces(forAge.getPCAFacesArray());
					vis.setPCAFlag(true);
					
					vis.setInitMeshFlag(false);
					vis.setFinalMeshFlag(false);
					
					viewInitMesh.setEnabled(false);
					viewTPSMesh.setEnabled(false);
					
					viewInitMesh.setSelected(false);
					viewTPSMesh.setSelected(false);
					viewCurvatureMesh.setEnabled(false);
					
					isFlippedFrame = new JFrame("Scan Orientation");
					isFlippedFrame.setSize(470,200);
					isFlippedFrame.setLocation(500, 500);
					isFlippedPane = new JEditorPane();
					Font font = new Font("TimesRoman", Font.BOLD, 14);
					isFlippedPane.setFont(font);
					
					isFlippedPane.setEditable(false);
					isFlippedPane.setText("\n If the ventral half of the pubic symphysis is pointing up\n select \"OK\" otherwise select to \"Flip\" it across the x-axis.");
					okButton = new JButton("OK");
					isFlippedPane.add(okButton);
					flipButton = new JButton("Flip");
					isFlippedPane.add(flipButton);

					Insets insets = pane.getInsets();
					Dimension size = okButton.getPreferredSize();
					okButton.setBounds(150 + insets.left, 70 + insets.top, size.width, size.height);
					size = flipButton.getPreferredSize();
					flipButton.setBounds(250 + insets.left, 70 + insets.top, size.width, size.height);
					
					isFlippedFrame.setContentPane(isFlippedPane);
					
					isFlippedFrame.setVisible(true);
					
					okButton.addActionListener(new ActionListener() {
						public void actionPerformed(ActionEvent e1) {
							if(e1.getSource() == okButton) 
							{
								isFlippedFrame.setVisible(false);
								forAge.doCalculations();
								
								vis.setInitMesh(forAge.getInitMeshArray());
								vis.setFinalMesh(forAge.getTPSMeshArray());
								vis.setCurvatureMesh(forAge.getCurvatureMeshArray());
								
								fileSave.setEnabled(true);
								viewPCAMesh.setEnabled(true);
								viewPCAMesh.setSelected(true);
								viewInitMesh.setEnabled(true);
								viewTPSMesh.setEnabled(true);
								viewCurvatureMesh.setEnabled(true);
								
								String str = ("You are using file: \n" + filePath + "\n\n");

								str += ("\n\n Age Estimate per Model\n");
								str += ("-------------------------------------------------------------------------------------------- \n");
								str += " SAH \t BE \t VC \tVC & SAH\tVC & BE \n";
								str += (" " + fmt.format(forAge.getSAHAge()) + "\t" + fmt.format(forAge.getBEAge()) + "\t" + fmt.format(forAge.getVCAge()));
								str += ("\t" + fmt.format(forAge.getSAHandVCAge()) + "\t" + fmt.format(forAge.getBEandVCAge()));
								
								str += ("\n\n\n\n Score Values\n");
								str += ("-------------------------------------------------------------------------------------------- \n");
								str += " SAH \t BE \t VC \n";
								str += (" " + fmt1.format(forAge.getSAH()) + "\t" + fmt1.format(forAge.getBE()) + "\t" + fmt1.format(forAge.getVC()));
								
								pane.setText(str);
							}
						}
					});
					
					flipButton.addActionListener(new ActionListener() {
						public void actionPerformed(ActionEvent e2) {
							if(e2.getSource() == flipButton) {
								forAge.getBoneScan().flipY();
								vis.setPCAVertices(forAge.getPCAVerticesArray());
								vis.setPCAFaces(forAge.getPCAFacesArray());
							}
						}
					 });
				}
			} 
			
			else if (option == savePCA)
			{
				jfc.setSelectedFile(new File("*.ply"));
				jfc.setFileFilter(fileTypePLY);
				int file = jfc.showOpenDialog(pane);
				if (file == JFileChooser.APPROVE_OPTION) 
				{
					File selectedFile = jfc.getSelectedFile();
					FileOutputStream fos = new FileOutputStream(selectedFile);
					OutputStreamWriter out = new OutputStreamWriter(fos, Charset.forName("UTF-8")); 
					out.write(forAge.getPCAString());
					out.close();
				}
			}
			else if (option == saveCurvatureMesh)
			{
				jfc.setSelectedFile(new File("*.obj"));
				jfc.setFileFilter(fileTypeOBJ);
				int file = jfc.showOpenDialog(pane);
				
				if (file == JFileChooser.APPROVE_OPTION) 
				{
					File selectedFile = jfc.getSelectedFile();
					FileOutputStream fos = new FileOutputStream(selectedFile);
					OutputStreamWriter out = new OutputStreamWriter(fos, Charset.forName("UTF-8")); 
					out.write(forAge.getCurvatureMeshString());
					out.close();
				}
			}
			else if (option == saveTPSMesh)
			{
				jfc.setSelectedFile(new File("*.obj"));
				jfc.setFileFilter(fileTypeOBJ);
				int file = jfc.showOpenDialog(pane);
				
				if (file == JFileChooser.APPROVE_OPTION) 
				{
					File selectedFile = jfc.getSelectedFile();
					FileOutputStream fos = new FileOutputStream(selectedFile);
					OutputStreamWriter out = new OutputStreamWriter(fos, Charset.forName("UTF-8")); 
					out.write(forAge.getTPSMeshString());
					out.close();
				}
			}
			else if ( option == viewPCAMesh )
			{
				if ( viewPCAMesh.isSelected() )
				{
					vis.setPCAFlag(true);
				}
				else
				{
					vis.setPCAFlag(false);
				}
			}
			else if ( option == viewInitMesh )
			{
				if ( viewInitMesh.isSelected() )
				{
					vis.setInitMeshFlag(true);
				}
				else
				{
					vis.setInitMeshFlag(false);
				}
			}
			else if ( option == viewTPSMesh )
			{
				if ( viewTPSMesh.isSelected() )
				{
					vis.setFinalMeshFlag(true);
				}
				else
				{
					vis.setFinalMeshFlag(false);
				}
			}
			else if ( option == viewCurvatureMesh )
			{
				if ( viewCurvatureMesh.isSelected() )
				{
					vis.setCurvatureMeshFlag(true);
				}
				else
				{
					vis.setCurvatureMeshFlag(false);
				}
			}
			else if (option == about)
			{	
				aboutFrame = new JFrame("About forAge");
				aboutFrame.setSize(400,200);
				aboutPane = new JEditorPane();
				aboutPane.setBackground(Color.lightGray);
				Font font = new Font("TimesRoman", Font.BOLD, 14);
				aboutPane.setFont(font);
				
				aboutPane.setEditable(false);
				
				String str = "\n forAge_20180217 2015 - Stoyanova, Algee-Hewitt, and Slice";
				str += "\n\n\n Developed by Detelina Stoyanova";
				
				aboutPane.setText(str);
				
				aboutFrame.setContentPane(aboutPane);
				
				aboutFrame.setVisible(true);
			}
			
		} catch (Exception f) {
			f.printStackTrace();
		}
		
	}

	@Override
	public void itemStateChanged(ItemEvent e) {
		// TODO Auto-generated method stub
		
	}
}
